unit TemplateUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Math;

type
TColumn = record
Initial, Final: TColor;
Left, Right: Integer;
end;
TPrefixSize = 1..SizeOf(ShortString);
  TTemplate = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
FMainCode, FManual: Boolean;
FElSize: Integer;
FPerPtr: PByte;
FPerCnt: Integer;
FSubNum: Word;
procedure PaintColumn(Column: TColumn);
function FileName: string;
    procedure setPer(Size: Word; const Value: Pointer);
    procedure SetElSize(const Value: Byte);
    function GetByName(const Prefix; PrefixSize: TPrefixSize;
      ElemSize: Byte): Pointer;
    procedure SetSubStream(const Value: Word);
    function GetSuccSub: Word;
protected
Columns: array of TColumn;
function CoolMB(Text, Cap: string; Flags: Integer): Integer;
procedure CriticalStop(Reason: string);
procedure Load;
procedure Save;
property ElementSize: Byte write SetElSize;
property Persistent[Size: Word]: Pointer write setPer;
procedure AppendElement(const E);
  public
    { Public declarations }
procedure ProcessIdle;
property Substream: Word read GetSuccSub write SetSubStream;
property ByPrefix[const Prefix; PrefixSize: TPrefixSize;
ElemSize: Byte]: Pointer read GetByName;
  end;

var
  Template: TTemplate;

implementation

{$R *.dfm}

procedure ConstExtract(Inp: TColor; var R, G, B: Byte);
var Color: TColor;
                           begin
with TBitmap.Create do try
Width := 10;
Height := 10;
Canvas.Pixels[5, 5] := Inp;
Color := Canvas.Pixels[5, 5];
R := GetRValue(Color);
G := GetGValue(Color);
B := GetBValue(Color);
finally Free; end;
end;

{ TTemplate }

procedure TTemplate.AppendElement(const E);
var HighElement: Cardinal;
SecondElement: PByte;
Dest: Pointer;
Z: AnsiString;
begin
Z := StringOfChar(#0, FElSize);
Dest := ByPrefix[Z[1], Length(Z), 0];
HighElement := Cardinal(FPerPtr);
Inc(HighElement, FPerCnt - FElSize); 
if Dest = Pointer(HighElement) then begin
SecondElement := FPerPtr;
Inc(SecondElement, FElSize);
Move(SecondElement^, FPerPtr^, Cardinal(HighElement) - Cardinal(FPerPtr));
                                                                          end;
Move(E, Dest^, SizeOf(Dest^));
end;

function TTemplate.GetByName;
var Ptr: PByte absolute Result;
NullStr: AnsiString;
function IsNull: Boolean;
begin
Result :=  CompareMem(@NullStr[1], Ptr, Length(NullStr)) ;
end;
              function Match: Boolean;
begin
Assert(ElemSize > 0);
Result := CompareMem(Ptr, @Prefix, PrefixSize);
if not Result then
Inc(Ptr, ElemSize);
end;
var I, Count: Integer;
begin
NullStr := StringOfChar(#0, PrefixSize);
Result := FPerPtr;
if ElemSize > 0 then begin // Has offset.
Assert(FElSize = -1);
             Count := Ptr^;
Inc(Ptr);
for I := 0 to Count-1 do
if Match then Exit;
end else  repeat // Null-terminated.
ElemSize := FElSize;
if Match then Exit;
until IsNull;
Result := nil;
end; 

procedure TTemplate.SetElSize(const Value: Byte);
begin
Assert(FElSize = 0); // 'ElementSize can only be called once: and it must be before Persistent is set.'
                                                                                                       FElSize := Value;
end;

procedure TTemplate.setPer;
begin
Assert(FPerPtr = nil); // Persistent can only be set once.
if FElSize = 0 then
FElSize := -1;
FPerPtr := Value;
FPerCnt := Size;
if FElSize > 0 then
Dec(FPerCnt, FElSize);
end;

procedure TTemplate.FormCreate(Sender: TObject);
begin
Canvas.Font.Assign(Font);
FMainCode := True;
Load;
end;

procedure TTemplate.SetSubStream;
begin
FSubNum := Value;
end;

function TTemplate.GetSuccSub: Word;
begin
Result := Succ(FSubNum);
end;

procedure TTemplate.Load;
begin
if not FMainCode then
FManual := True;
if FileExists(FileName) then
with TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite) do try
ReadBuffer(FPerPtr^, FPerCnt);
finally Free; end else
FillChar(FPerPtr^, FPerCnt, #0);
end;

procedure TTemplate.Save;
begin

end;

procedure TTemplate.ProcessIdle;
begin
if FManual then Exit;
Save;
Load;
end;

procedure TTemplate.FormDestroy(Sender: TObject);
begin
if Assigned(FPerPtr) then
with TFileStream.Create(FileName, fmCreate) do try
WriteBuffer(FperPtr^, FPerCnt);
finally Free; end;
end;

function TTemplate.FileName: string;
begin
Result := ExtractFilePath(
Application.ExeName) +
'Database' + PathDelim;
if not DirectoryExists(Result) then CriticalStop(
'The following directory does not exist:' + sLineBreak +
Result + sLineBreak +
'Please create it -- it can just be an empty directory.');
Result := Result + Name;
Insert('.', Result, Length(Result) - 3);
if FSubNum > 0 then
Result := Result + ':S' + IntToStr(FSubNum);
end;

procedure TTemplate.PaintColumn;
type TRange = array[Boolean] of Byte;
const Max = 7;
var R, G, B: TRange;
Rec: TRect;
Y: Integer;
function P(C: TRange): Byte;
begin
Result := EnsureRange(
(((C[false] - C[true]) * Y) div Max) + c[true],
Low(Byte), High(Byte));
end;
begin
  inherited;
ConstExtract(Column.Initial, R[false], G[false], B[false]);
ConstExtract(Column.Final, R[true], g[true], b[true]);
Rec.Left := Column.Left;
Rec.Right := Column.Right;
for Y := 0 to Max do begin
with Rec do begin
Bottom := ClientHeight div Succ(Max);
Top := Bottom * Y;
Inc(Bottom, Top);
end;
Canvas.Brush.Color := RGB(P(R), P(G), P(B));
Canvas.FillRect(Rec);
    end; end;

procedure TTemplate.FormPaint(Sender: TObject);
var I: Integer;
begin
inherited;
for I := 0 to High(Columns) do
PaintColumn(Columns[i]);
end;

procedure TTemplate.CriticalStop(Reason: string);
begin
Reason := Reason + sLineBreak + sLineBreak +
'Press OK to quit.';
CoolMB(Reason,
'Critical Stop',
MB_IconWarning);
Application.Terminate;
Halt(2);
end;

function TTemplate.CoolMB(Text, Cap: string;
  Flags: Integer): Integer;
var H: HWND;
begin
if Visible then
H := Handle
else H := Application.Handle;
Result := MessageBox(H, PChar(Text), PChar(Cap), Flags);
end;

end.
