unit XmlConst;

interface

uses SysUtils;

const
XML = '.xml';
XmlUTF8 = '<?xml version="1.0" encoding="UTF-8"?>';

type TSecondIndex = 1..High(Byte);

{ Write functions: }

function ValueStart(Name: string; Count: Byte; First: string = ''): string;
function SecondValue(More: string; Index: TSecondIndex): string;
function ValueEnd(Name: string): string;

type TAlgoCount = (agIncorrect, agZero, agSingle, agMultiple);

{ Read functions: }

function ExtractCount(Line, Name: string): TAlgoCount;
function HasValue(Line: string): Boolean;
function ExtractValue(Line: string): string;

implementation

procedure Sanitize(var S: string);
var I: Integer;
begin
UniqueString(S);
for I := 1 to Length(S) do
case S[i] of
'"': S[I] := '''';
'<', '>', #0: S[i] := '#';
end;
end;

function ValueStart;
begin
Sanitize(First);
Result := '<' + Name + ' ';
case Count of
0:Result := Result + 'no="elements"/';
1: Result := Result + '_="' + First + '"/';
else Result := Result + 'multiple="parts"';
                                           end;
Result := Result + '>';
end;

function SecondValue;
begin
Sanitize(More);
Result := '<_' + IntToStr(Index) + ' _="' + More + '"/>';
end;

function ValueEnd;
begin
Result := '</' + Name + '>';
end;

function ExtractCount;
                     begin
Result := agIncorrect;
if Pos(Name + ' ', Line) <> 2 then Exit;
Result := agZero;
if Pos('o="e', Line) > 0 then Exit;
Result := agMultiple;
if Pos('e="p', Line) > 0 then Exit;
Result := agSingle;
           end;

function HasValue;
begin
Result := Pos('"', Line) > 0;
end;

function ExtractValue;
begin
Delete(Line, 1, Pos('"', Line));
Delete(Line, Pos('"', Line), Length(Line));
Result := Line;
end;

end.
